import React, { Component } from 'react';
import { AuctionList } from './AuctionList';
import { AuctionDetails } from './AuctionDetails';
import { NewAuction, NewAuctionInfo } from './NewAuction';
import { Auction } from './auction';


// Indicates which page to show. If it is the details page, the argument
// includes the specific auction to show the details of.
type Page = "list" | "new" | {kind: "details", index: number}; 

type AppState = {page: Page, auctions: ReadonlyArray<Auction>};

// Whether to show debugging information in the console.
const DEBUG: boolean = true;

// Top-level component that displays the appropriate page.
export class App extends Component<{}, AppState> {

  constructor(props: {}) {
    super(props);
    this.state = {page: "list", auctions: []};
  }

  render = (): JSX.Element => {
    if (this.state.page === "list") {
      if (DEBUG) console.debug("rendering list page");
      return <AuctionList auctions={this.state.auctions}
                          onNewClick={this.doNewClick}
                          onAuctionClick={this.doAuctionClick}/>;

    } else if (this.state.page === "new") {
      if (DEBUG) console.debug("rendering add page");
      return <NewAuction onStartClick={this.doStartClick}
                         onBackClick={this.doBackClick}/>;

    } else {  // details
      const index = this.state.page.index;
      const auction = this.state.auctions[index];
      if (DEBUG) console.debug(`rendering details page for "${auction.name}"`);
      return <AuctionDetails auction={auction}
                  onBidClick={(bidder, amt) => this.doBidClick(index, bidder, amt)}
                  onBackClick={this.doBackClick}/>;
    }
  };

  doNewClick = (): void => {
    if (DEBUG) console.debug("set state to new");
    this.setState({page: "new"});
  };
  
  doAuctionClick = (index: number): void => {
    const auction = this.state.auctions[index];
    if (DEBUG) console.debug(`set state to details for "${auction.name}"`);
    this.setState({page: {kind: "details", index}});
  };
  
  doStartClick = (info: NewAuctionInfo): void => {
    if (DEBUG) console.debug("add an auction and then show the full list");

    // We will act as if the seller bid just below the min bid. That requires
    // someone else to bid at least the min bid to win the auction.
    const maxBid = info.minBid - 1;
    const maxBidder = info.seller;

    // Record the time since epoch when this ends.
    const endTime = Date.now() + info.minutes * 60 * 1000;

    const auction = {
        name: info.name, description: info.description, seller: info.seller,
        maxBid, maxBidder, endTime
      };
    const auctions = this.state.auctions.concat([auction]);
    this.setState({page: "list", auctions});
  };
  
  doBackClick = (): void => {
    if (DEBUG) console.debug("set state to list");
    this.setState({page: "list"});
  };

  doBidClick = (index: number, bidder: string, amount: number): void => {
    const oldVal = this.state.auctions[index];
    const newVal = {
        name: oldVal.name, seller: oldVal.seller,
        description: oldVal.description, endTime: oldVal.endTime,
        maxBidder: bidder, maxBid: amount
      };

    // Re-display the details page with the new auction.
    const auctions = this.state.auctions.slice(0, index)
        .concat([newVal])
        .concat(this.state.auctions.slice(index+1));
    this.setState({auctions, page: {kind: "details", index}});
  };
}