import React, { Component, ChangeEvent, MouseEvent } from 'react';
import { Auction } from './auction';


type DetailsProps = {
  auction: Auction,
  onBidClick: (bidder: string, amount: number) => void,
  onBackClick: () => void
};

type DetailsState = {
  now: number,
  bidder: string,
  amount: string,
  error: string
};


// Shows an individual auction and allows bidding (if ongoing).
export class AuctionDetails extends Component<DetailsProps, DetailsState> {

  constructor(props: DetailsProps) {
    super(props);

    const amount = this.props.auction.maxBid + 1;  // min bid is 1 more
    this.state = {now: Date.now(), bidder: "", amount: '' + amount, error: ""};
  }

  componentDidUpdate = (prevProps: DetailsProps): void => {
    if (prevProps !== this.props) {
      // If the user's bid is too small, update it to be valid. In any case,
      // we need to update the time.
      const amount = parseFloat(this.state.amount);
      if (!isNaN(amount) && amount < this.props.auction.maxBid + 1) {
        this.setState({amount: '' + (this.props.auction.maxBid + 1),
                       now: Date.now(), error: ""});
      } else {
        this.setState({now: Date.now(), error: ""});
      }
    }
  };

  render = (): JSX.Element => {
    const auction = this.props.auction;
    if (auction.endTime <= this.state.now) {
      return this.renderCompleted();
    } else {
      return this.renderOngoing();
    }
  };

  renderCompleted = (): JSX.Element => {
    const auction = this.props.auction;
    return (
      <div>
        <h2>{auction.name}</h2>
        <p>{auction.description}</p>
        <p>Winning Bid: {auction.maxBid} (by {auction.maxBidder})</p>
      </div>);
  };

  renderOngoing = (): JSX.Element => {
    const auction = this.props.auction;
    const min = Math.round((auction.endTime - this.state.now) / 60 / 100) / 10;
    return (
      <div>
        <h2>{auction.name}</h2>
        <p>{auction.description}</p>
        <p><i>Bidding ends in {min} minutes...</i></p>
        <p>Current Bid: ${auction.maxBid} by {auction.maxBidder}</p>
        <div>
          <label htmlFor="bidder">Name:</label>
          <input type="text" id="bidder" value={this.state.bidder} 
              onChange={this.doBidderChange}></input>
        </div>
        <div>
          <label htmlFor="amount">Amount:</label>
          <input type="number" min={auction.maxBid + 1}
              id="amount" value={this.state.amount} 
              onChange={this.doAmountChange}></input>
        </div>
        <button type="button" onClick={this.doBidClick}>Bid</button>
        <button type="button" onClick={this.doRefreshClick}>Refresh</button>
        <button type="button" onClick={this.doDoneClick}>Done</button>
        {this.renderError()}
      </div>);
  };

  renderError = (): JSX.Element => {
    if (this.state.error.length === 0) {
      return <div></div>;
    } else {
      const style = {width: '300px', backgroundColor: 'rgb(246,194,192)',
          border: '1px solid rgb(137,66,61)', borderRadius: '5px', padding: '5px' };
      return (<div style={{marginTop: '15px'}}>
          <span style={style}><b>Error</b>: {this.state.error}</span>
        </div>);
    }
  };

  doBidderChange = (evt: ChangeEvent<HTMLInputElement>): void => {
    this.setState({bidder: evt.target.value, error: ""});
  };

  doAmountChange = (evt: ChangeEvent<HTMLInputElement>): void => {
    this.setState({amount: evt.target.value, error: ""});
  };

  doBidClick = (_: MouseEvent<HTMLButtonElement>): void => {
    // Verify that the user entered all required information.
    if (this.state.bidder.trim().length === 0 ||
        this.state.amount.trim().length === 0) {
      this.setState({error: "a required field is missing."});
      return;
    }

    // Verify that amount is a positive integer.
    const amount = parseFloat(this.state.amount);
    if (isNaN(amount) || Math.floor(amount) !== amount) {
      this.setState({error: "amount is not an integer"});
      return;
    }

    // Verify that amount is bigger than the current bid.
    if (amount < this.props.auction.maxBid + 1) {
      this.setState({error: "amount is not bigger than current bid"});
      return;
    }

    this.props.onBidClick(this.state.bidder, amount);
  };

  doRefreshClick = (_evt: MouseEvent<HTMLButtonElement>): void => {
    // Update the time at which we are showing 
    this.setState({now: Date.now(), error: ""});
  };

  doDoneClick = (_: MouseEvent<HTMLButtonElement>): void => {
    this.props.onBackClick();  // tell the parent to show the full list again
  };
}
